"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _uuid = _interopRequireDefault(require("uuid"));

var _http = _interopRequireDefault(require("http"));

var _https = _interopRequireDefault(require("https"));

var _express = _interopRequireDefault(require("express"));

var _bodyParser = _interopRequireDefault(require("body-parser"));

var _cors = _interopRequireDefault(require("cors"));

var _geoipLite = _interopRequireDefault(require("geoip-lite"));

var _useragent = _interopRequireDefault(require("useragent"));

var _ws = _interopRequireDefault(require("ws"));

var _jsonwebtoken = _interopRequireDefault(require("jsonwebtoken"));

var _nodeIpa = _interopRequireDefault(require("@obl/node-ipa"));

var _configYml = _interopRequireDefault(require("config-yml"));

var _configureStore = _interopRequireDefault(require("./store/configureStore"));

var appInfo = _interopRequireWildcard(require("./actions/appInfo"));

var wsBroker = _interopRequireWildcard(require("./actions/wsBroker"));

var _zmqBroker = require("./selectors/zmqBroker");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var jwt_secret = _uuid["default"].v4();

console.log("jwt_secret : " + jwt_secret);
var app = (0, _express["default"])();

if (_configYml["default"].zmq2ws.ldap.url.length === 0 || _configYml["default"].zmq2ws.ldap.domain.length === 0 || jwt_secret === 0) {
  console.log("Warning : No auth setup");
} else {
  console.log("Auth: " + _configYml["default"].zmq2ws.ldap.domain);

  _nodeIpa["default"].create(_configYml["default"].zmq2ws.ldap.url, _configYml["default"].zmq2ws.ldap.domain);
}

app.use(_bodyParser["default"].json());
app.use(_bodyParser["default"].urlencoded({
  extended: false
}));
app.use((0, _cors["default"])({
  allowedOrigins: '*'
}));
app.disable('x-powered-by');

if (_configYml["default"].zmq2ws.ssl.enabled) {
  var privateKey = _fs["default"].readFileSync(_configYml["default"].zmq2ws.ssl.key, 'utf8');

  var certificate = _fs["default"].readFileSync(_configYml["default"].zmq2ws.ssl.cert, 'utf8');

  var credentials = {
    key: privateKey,
    cert: certificate
  };
  app.server = _https["default"].createServer(credentials, app);
  app.set('port', _configYml["default"].zmq2ws.port);
} else {
  app.server = _http["default"].createServer(app);
  app.set('port', _configYml["default"].zmq2ws.port);
}

app.wss = new _ws["default"].Server({
  server: app.server
});

app.wss.broadcast = function broadcast(data) {
  app.wss.clients.forEach(function each(client) {
    if (client.readyState === _ws["default"].OPEN) {
      client.send(data);
    }
  });
};

app.wss.on('connection', function (ws, req) {
  // generate id of websocket
  ws.id = _uuid["default"].v4(); // get number of clients

  var nclients = Array.from(app.wss.clients).length;
  console.log('[' + nclients + '] User ' + ws.id + ' was connected');
  var ip = req.headers['x-real-ip'] || req.connection.remoteAddress || null;
  var geo = ip ? _geoipLite["default"].lookup(ip) : null; // get ip and user info

  var userInfo = {
    id: ws.id,
    ip: ip,
    geo: geo,
    info: _useragent["default"].lookup(req.headers['user-agent']).toJSON(),
    username: ""
  };

  _configureStore["default"].dispatch(appInfo.onWebSocketEnter(app.wss, userInfo));

  var wsInfo = {
    type: "ws",
    payload: {
      id: ws.id
    }
  };
  ws.send(JSON.stringify(wsInfo));
  ws.on('message', function (msg) {
    console.log('received: %s', msg);
    var o = JSON.parse(msg);
    var n = "";
    if (o.src === undefined) o.src = "obmon";

    if (o.type === "sub") {
      var b = (0, _zmqBroker.getZeroMQBroker)(_configureStore["default"].getState()).zmqs$;
      n = o.src + "|" + o.name + "|" + o.sub;

      if (b[n] !== undefined) {
        if (b[n].zmq$ !== undefined) {
          _configureStore["default"].dispatch(wsBroker.onWebSocketSub(ws, b[n].zmq$, n));
        }
      } else {
        console.log("Ws:onMesage => We could not find service : " + n);
      }
    } else if (o.type === "unsub") {
      n = o.src + "|" + o.name + "|" + o.sub;

      _configureStore["default"].dispatch(wsBroker.onWebSocketUnSub(ws, n));
    } else if (o.type === "updateinfo") {
      console.log("Info update for ws " + ws.id + " username: " + o.username + " uid: " + o.uid);

      _configureStore["default"].dispatch(appInfo.onWebSocketUserLogin(app.wss, ws.id, o.username, o.username));
    } else {// ws.send(JSON.stringify({ type: "zmq", src: "" }));
    }
  });
  ws.on('close', function () {
    _configureStore["default"].dispatch(appInfo.onWebSocketExit(app.wss, {
      id: ws.id
    }));

    var nclients = Array.from(app.wss.clients).length;
    console.log('[' + nclients + '] User ' + ws.id + ' was disconnected');

    _configureStore["default"].dispatch(wsBroker.onWebSocketUnSub(ws, ""));
  });
  ws.on('error', function () {
    return console.log('Error from web socket');
  });
});
app.post('/api/login', function (req, res) {
  if (_configYml["default"].zmq2ws.ldap.url.length === 0 || _configYml["default"].zmq2ws.ldap.domain.length === 0 || jwt_secret.length === 0) {
    res.status(400);
    res.send("No auth setup on API !!!");
    return;
  }

  var username = req.body.username;
  var password = req.body.password;
  var wsid = req.body.wsid || "";

  _nodeIpa["default"].login(username, password, function (rc, profile) {
    res.setHeader('Content-Type', 'application/json');

    if (rc) {
      var token = _jsonwebtoken["default"].sign({
        profile: profile
      }, jwt_secret, {
        expiresIn: '1d' // expires in 24 hours

      });

      console.log(JSON.stringify(profile));
      if (wsid !== "") _configureStore["default"].dispatch(appInfo.onWebSocketUserLogin(app.wss, wsid, profile.uid, profile.uidNumber));
      res.status(200).send({
        auth: true,
        token: token
      });
    } else {
      res.status(403);
      res.send("Invalid credentials");
    }
  });
});
app.get('*', function (req, res) {
  res.sendFile(__dirname + '/index.html');
});
var _default = app;
exports["default"] = _default;