let ldap = require("ldapjs");
let ldap2date = require("ldap2date");

config = {
  ldap: {
    url: "",
    realm: "",
    base: "",
    users: "",
    groups: ""
  },
  jwt: {
    secret: ""
  }
};

exports.create = function(url, domain) {
  config.ldap.url = url;
  config.ldap.realm = domain;
  let base = domain.split(".");
  config.ldap.base = "cn=accounts";
  for (let i = 0; i < base.length; i++) {
    config.ldap.base += ", dc=" + base[i];
  }
  config.ldap.users = "cn=users," + config.ldap.base;
  config.ldap.groups = "cn=groups, " + config.ldap.base;
};

exports.login = function(username, password, callback) {
  let dn = "uid=" + username + "," + config.ldap.users;
  let ldapclient = ldap.createClient({
    url: config.ldap.url,
    tlsOptions: {
      rejectUnauthorized: false
    }
  });
  ldapclient.bind(dn, password, function(err) {
    if (err) {
      //console.log('\nAuthorization failure!\nLogin or password incorrect');
      ldapclient.unbind();
      callback(false, err);
    } else {
      let opts = {};
      ldapclient.search(dn, opts, function(err, ldapres) {
        ldapres.on("searchEntry", function(entry) {
          let obj = entry.object;
          // filtring groups
          let group_members = [];
          if (obj.memberOf.constructor === Array) group_members = obj.memberOf;
          else group_members.push(obj.memberOf);
          let groups = group_members.filter(function(arrayElement) {
            return (
              arrayElement.indexOf(config.ldap.base) ==
              arrayElement.length - config.ldap.base.length
            );
          });
          if (obj.krbLastSuccessfulAuth)
            obj.krbLastSuccessfulAuth = ldap2date.parse(
              obj.krbLastSuccessfulAuth
            );
          if (obj.krbLastFailedAuth)
            obj.krbLastFailedAuth = ldap2date.parse(obj.krbLastFailedAuth);

          // creating profile
          let profile = {
            dn: obj.dn,
            ipaUniqueID: obj.ipaUniqueID,
            krbPrincipalName: obj.krbPrincipalName,
            uid: obj.uid,
            displayName: obj.displayName,
            mobile: obj.mobile,
            mail: obj.mail,
            uidNumber: obj.uidNumber,
            gidNumber: obj.gidNumber,
            groups: groups,
            loginShell: obj.loginShell,
            homeDirectory: obj.homeDirectory,
            krbPasswordExpiration: ldap2date.parse(obj.krbPasswordExpiration),
            krbLastPwdChange: ldap2date.parse(obj.krbLastPwdChange),
            krbExtraData: obj.krbExtraData,
            krbLastSuccessfulAuth: obj.krbLastSuccessfulAuth,
            krbLastFailedAuth: obj.krbLastFailedAuth,
            krbLoginFailedCount: obj.krbLoginFailedCount
          };

          callback(true, profile);
        });
        ldapres.on("searchReference", function(referral) {
          console.log("referral: " + referral.uris.join());
          ldapclient.unbind();
          callback(false, {});
        });
        ldapres.on("error", function(err) {
          console.error("error: " + err.message);
          ldapclient.unbind();
          callback(false, {});
        });
        ldapres.on("end", function(result) {
          console.log(
            'ldap: User "' + username + '" login with status ' + result.status
          );
          ldapclient.unbind();
        });
      });
    }
  });
};
